#!/usr/bin/php
<?php

	// Thomas Radeke, 2016-10
	
	// This script takes a bunch of images, calculates their absolute EV and brightness
	// level, then passes them to Luminance HDR for making an HDR image.
	// 
	// The script is made to be called from the context menu of either Caja or Nautilus.
	// To enable it, copy the .php file into
	// /home/username/.config/caja/scripts (or nautilus/scripts), make it executable
	// and install the package "php5". Note that PHP must be compiled with --enable-exif
	// and the exif library must be enabled in php.ini.
	//
	// After installing all dependencies, select some photos and call this script
	// from the context "Scripts" menu.

	// Function to calculate fractional values from strings like "1/6400"
	function calc($ma){
		$p = -9999;
		if(preg_match('/(\d+)(?:\s*)([\+\-\*\/])(?:\s*)(\d+)/', $ma, $matches) !== FALSE){
			$operator = $matches[2];

			switch($operator){
				case '+':
					$p = $matches[1] + $matches[3];
					break;
				case '-':
					$p = $matches[1] - $matches[3];
					break;
				case '*':
					$p = $matches[1] * $matches[3];
					break;
				case '/':
					$p = $matches[1] / $matches[3];
					break;
			}
		}
		return $p;
	}

	// Prepare selected file array
	$selected_paths = array();
	
	// Get selected files from Caja/Nautilus environment variables
	if(isset($_SERVER["NAUTILUS_SCRIPT_SELECTED_FILE_PATHS"])){
		$selected_paths = trim($_SERVER["NAUTILUS_SCRIPT_SELECTED_FILE_PATHS"]);
		$selected_paths = explode("\n", $selected_paths);
	}
	
	// Open a debug.txt file for debugging purposes, because there's no console output
	// when running a script from the context menu in Caja/Nautilus
	$output = fopen("debug.txt", "w");

	// We need a sensible default EV
	$base_ev = 8;
	
	// Get first and last filenames to generate an appropriate output name
	$ev_values = array();
	$first_path = reset($selected_paths);
	$last_path = end($selected_paths);
	
	$first_path_parts = pathinfo($first_path);
	$last_path_parts = pathinfo($last_path);
	
	$first_file = $first_path_parts["filename"];
	$last_file = $last_path_parts["filename"];
	$output_filename = "{$first_file}-{$last_file}.exr";
	
	// Now, calculate individual EVs from EXIF data (requires PHP EXIF library)
	foreach($selected_paths as $fullpath){
	
		// First, get relevant EXIF data
		$exif = exif_read_data($fullpath);
		$relevant = array(
			"ExposureTime" => $exif["ExposureTime"],
			"FNumber" => $exif["FNumber"],
			"ISO" => $exif["ISOSpeedRatings"],
			"ExposureCompensation" => $exif["ExposureBiasValue"]
		);
		
		// Calculate absolute EV
		// EV = log2 (fstop² / shutter time)
		$fstop = calc($relevant["FNumber"]);
		$shuttertime = calc($relevant["ExposureTime"]);
		$iso = $relevant["ISO"];
		$ev = log((($fstop*$fstop)/$shuttertime), 2);
				
		//Now calculate light value
		//LV = EV + log2 (ISO / 100)
		$lv = $ev + log($iso/100, 2);
		
		// Compensate overall brightness with base EV
		$comp_ev = $base_ev-$ev;
		
		$ev_values[] = $comp_ev;
		$quoted_paths[] = '"'.$fullpath.'"';
		fwrite($output, "\nFile: $fullpath\nEV: $ev ($comp_ev)\nLV: $lv\n(ExposureTime: $shuttertime, FStop: $fstop, ISO: $iso)\n\n");
	}
	
	// Generate command to run. Requires Luminance HDR to be installed and available in the path.
	$command = 'luminance-hdr-cli -e '.implode(",", $ev_values).' -c weight=gaussian:response_curve=gamma '.implode(" ", $quoted_paths).' -s "'.$output_filename.'"';
	fwrite($output, $command);
	exec($command);
	
	// We're done, close debug text file
	fclose($output);
?>